@@ -172,7 +172,7 @@ class Cluster:
         self.template_body = None
         self.__config = None
         self.__s3_artifact_dir = None
-
+        self.__official_ami = None
         self.__has_running_capacity = None
         self.__running_capacity = None
 
@@ -181,6 +181,7 @@ class Cluster:
         """Return the ClusterStack object."""
         if not self.__stack:
             self.__stack = ClusterStack(AWSApi.instance().cfn.describe_stack(self.stack_name))
+            self.__official_ami = self.__stack.official_ami
         return self.__stack
 
     @property
@@ -358,10 +359,12 @@ class Cluster:
                 validator_suppressors, validation_failure_level
             )
 
+            LOGGER.info("Generating artifact dir and uploading config...")
             self._add_tags()
             self._generate_artifact_dir()
             artifact_dir_generated = True
             self._upload_config()
+            LOGGER.info("Generation and upload completed successfully")
 
             # Create template if not provided by the user
             if not (self.config.dev_settings and self.config.dev_settings.cluster_template):
@@ -369,8 +372,10 @@ class Cluster:
                     cluster_config=self.config, bucket=self.bucket, stack_name=self.stack_name
                 )
 
+            LOGGER.info("Uploading cluster artifacts...")
             # upload cluster artifacts and generated template
             self._upload_artifacts()
+            LOGGER.info("Upload of cluster artifacts completed successfully")
 
             LOGGER.info("Creating stack named: %s", self.stack_name)
             creation_result = AWSApi.instance().cfn.create_stack_from_url(
@@ -432,6 +437,8 @@ class Cluster:
             LOGGER.info("Validating cluster configuration...")
             Cluster._load_additional_instance_type_data(cluster_config_dict)
             config = self._load_config(cluster_config_dict)
+            config.official_ami = self.__official_ami
+
             validation_failures = config.validate(validator_suppressors, context)
             if any(f.level.value >= FailureLevel(validation_failure_level).value for f in validation_failures):
                 raise ConfigValidationError("Invalid cluster configuration.", validation_failures=validation_failures)
@@ -504,6 +511,7 @@ class Cluster:
         All files contained in root dir will be uploaded to
         {bucket_name}/parallelcluster/{version}/clusters/{cluster_name}/{resource_dir}/artifact.
         """
+        LOGGER.info("Uploading cluster artifacts to S3...")
         self._check_bucket_existence()
         try:
             resources = pkg_resources.resource_filename(__name__, "../resources/custom_resources")
@@ -530,6 +538,7 @@ class Cluster:
 
             if isinstance(self.config.scheduling, SchedulerPluginScheduling):
                 self._render_and_upload_scheduler_plugin_template()
+            LOGGER.info("Cluster artifacts uploaded correctly.")
         except BadRequestClusterActionError:
             raise
         except Exception as e:
