@@ -125,7 +125,12 @@ from pcluster.constants import (
     SUPPORTED_OSES,
 )
 from pcluster.models.s3_bucket import parse_bucket_url
-from pcluster.schemas.common_schema import AdditionalIamPolicySchema, BaseDevSettingsSchema, BaseSchema
+from pcluster.schemas.common_schema import (
+    AdditionalIamPolicySchema,
+    BaseDevSettingsSchema,
+    BaseSchema,
+    DeploymentSettingsSchema,
+)
 from pcluster.schemas.common_schema import ImdsSchema as TopLevelImdsSchema
 from pcluster.schemas.common_schema import TagSchema, get_field_validator, validate_no_reserved_tag
 from pcluster.utils import yaml_load
@@ -318,6 +323,8 @@ class EfsSettingsSchema(BaseSchema):
     deletion_policy = fields.Str(
         validate=validate.OneOf(DELETION_POLICIES), metadata={"update_policy": UpdatePolicy.SUPPORTED}
     )
+    encryption_in_transit = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    iam_authorization = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
     @validates_schema
     def validate_file_system_id_ignored_parameters(self, data, **kwargs):
@@ -326,7 +333,7 @@ class EfsSettingsSchema(BaseSchema):
         messages = []
         if data.get("file_system_id") is not None:
             for key in data:
-                if key is not None and key != "file_system_id":
+                if key is not None and key not in ["encryption_in_transit", "iam_authorization", "file_system_id"]:
                     messages.append(EFS_MESSAGES["errors"]["ignored_param_with_efs_fs_id"].format(efs_param=key))
             if messages:
                 raise ValidationError(message=messages)
@@ -645,18 +652,18 @@ class PlacementGroupSchema(BaseSchema):
 class QueueNetworkingSchema(BaseNetworkingSchema):
     """Represent the schema of the Networking, child of Queue."""
 
-    subnet_ids = fields.List(
-        fields.Str(validate=get_field_validator("subnet_id")),
-        required=True,
-        validate=validate.Length(equal=1),
-        metadata={"update_policy": UpdatePolicy.MANAGED_FSX},
-    )
     assign_public_ip = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
 
 class SlurmQueueNetworkingSchema(QueueNetworkingSchema):
     """Represent the schema of the Networking, child of slurm Queue."""
 
+    subnet_ids = fields.List(
+        fields.Str(validate=get_field_validator("subnet_id")),
+        required=True,
+        validate=validate.Length(min=1),
+        metadata={"update_policy": UpdatePolicy.MANAGED_FSX},
+    )
     placement_group = fields.Nested(
         PlacementGroupSchema, metadata={"update_policy": UpdatePolicy.MANAGED_PLACEMENT_GROUP}
     )
@@ -671,6 +678,13 @@ class SlurmQueueNetworkingSchema(QueueNetworkingSchema):
 class AwsBatchQueueNetworkingSchema(QueueNetworkingSchema):
     """Represent the schema of the Networking, child of aws batch Queue."""
 
+    subnet_ids = fields.List(
+        fields.Str(validate=get_field_validator("subnet_id")),
+        required=True,
+        validate=validate.Length(equal=1),
+        metadata={"update_policy": UpdatePolicy.MANAGED_FSX},
+    )
+
     @post_load
     def make_resource(self, data, **kwargs):
         """Generate resource."""
@@ -829,6 +843,8 @@ class ClusterIamSchema(BaseSchema):
         metadata={"update_policy": UpdatePolicy.SUPPORTED}, validate=validate.Regexp("^arn:.*:policy/")
     )
 
+    resource_prefix = fields.Str(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
     @post_load
     def make_resource(self, data, **kwargs):
         """Generate resource."""
@@ -1050,11 +1066,26 @@ class HeadNodeCustomActionSchema(BaseSchema):
         return CustomAction(**data)
 
 
+class HeadNodeUpdatableCustomActionSchema(BaseSchema):
+    """Represent the schema of an updatable custom action."""
+
+    script = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    args = fields.List(fields.Str(), metadata={"update_policy": UpdatePolicy.SUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return CustomAction(**data)
+
+
 class HeadNodeCustomActionsSchema(BaseSchema):
     """Represent the schema for all available custom actions."""
 
     on_node_start = fields.Nested(HeadNodeCustomActionSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     on_node_configured = fields.Nested(HeadNodeCustomActionSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    on_node_updated = fields.Nested(
+        HeadNodeUpdatableCustomActionSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED}
+    )
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1112,7 +1143,7 @@ class HeadNodeSchema(BaseSchema):
     ssh = fields.Nested(SshSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
     local_storage = fields.Nested(HeadNodeStorageSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
     dcv = fields.Nested(DcvSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
-    custom_actions = fields.Nested(HeadNodeCustomActionsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    custom_actions = fields.Nested(HeadNodeCustomActionsSchema, metadata={"update_policy": UpdatePolicy.IGNORED})
     iam = fields.Nested(HeadNodeIamSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
     imds = fields.Nested(ImdsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     image = fields.Nested(HeadNodeImageSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
@@ -1950,7 +1981,7 @@ class ClusterSchema(BaseSchema):
     tags = fields.Nested(
         TagSchema, many=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED, "update_key": "Key"}
     )
-    iam = fields.Nested(ClusterIamSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    iam = fields.Nested(ClusterIamSchema, metadata={"update_policy": UpdatePolicy.IGNORED})
     directory_service = fields.Nested(
         DirectoryServiceSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
     )
@@ -1959,6 +1990,7 @@ class ClusterSchema(BaseSchema):
     custom_s3_bucket = fields.Str(metadata={"update_policy": UpdatePolicy.READ_ONLY_RESOURCE_BUCKET})
     additional_resources = fields.Str(metadata={"update_policy": UpdatePolicy.SUPPORTED})
     dev_settings = fields.Nested(ClusterDevSettingsSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    deployment_settings = fields.Nested(DeploymentSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
     def __init__(self, cluster_name: str):
         super().__init__()
@@ -1973,11 +2005,18 @@ class ClusterSchema(BaseSchema):
     def no_settings_for_batch(self, data, **kwargs):
         """Ensure IntelSoftware and DirectoryService section is not included when AWS Batch is the scheduler."""
         scheduling = data.get("scheduling")
+        head_node = data.get("head_node")
         if scheduling and scheduling.scheduler == "awsbatch":
             error_message = "The use of the {} configuration is not supported when using awsbatch as the scheduler."
             additional_packages = data.get("additional_packages")
-            if additional_packages and additional_packages.intel_software.intel_hpc_platform:
+            if (
+                additional_packages
+                and additional_packages.intel_software
+                and additional_packages.intel_software.intel_hpc_platform
+            ):
                 raise ValidationError(error_message.format("IntelSoftware"))
+            if head_node.custom_actions and head_node.custom_actions.on_node_updated:
+                raise ValidationError(error_message.format("OnNodeUpdated"))
             if data.get("directory_service"):
                 raise ValidationError(error_message.format("DirectoryService"))
 
