@@ -20,7 +20,7 @@ import json
 from marshmallow import Schema, ValidationError, fields, post_dump, post_load, pre_dump, validate, validates
 
 from pcluster.config.cluster_config import BaseTag
-from pcluster.config.common import AdditionalIamPolicy, Cookbook, Imds
+from pcluster.config.common import AdditionalIamPolicy, Cookbook, DeploymentSettings, Imds, LambdaFunctionsVpcConfig
 from pcluster.config.update_policy import UpdatePolicy
 from pcluster.constants import PCLUSTER_PREFIX, SUPPORTED_ARCHITECTURES
 from pcluster.utils import to_pascal_case
@@ -87,7 +87,7 @@ class BaseSchema(Schema):
         """
         if kwargs.get("partial"):
             # If the schema is to be loaded partially, do not check existence constrain.
-            return True
+            return False
         num_of_fields = len([data.get(field_name) for field_name in field_list if data.get(field_name)])
         return num_of_fields != 1 if one_required else num_of_fields > 1
 
@@ -201,6 +201,41 @@ class CookbookSchema(BaseSchema):
             raise ValidationError(message="'{0}' is invalid".format(value))
 
 
+class LambdaFunctionsVpcConfigSchema(BaseSchema):
+    """Represent the VPC configuration schema of PCluster Lambdas, used both by build image and cluster files."""
+
+    security_group_ids = fields.List(
+        fields.Str(validate=get_field_validator("security_group_id")),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+        validate=validate.Length(min=1, max=5),
+        required=True,
+    )
+    subnet_ids = fields.List(
+        fields.Str(validate=get_field_validator("subnet_id")),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+        validate=validate.Length(min=1, max=16),
+        required=True,
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return LambdaFunctionsVpcConfig(**data)
+
+
+class DeploymentSettingsSchema(BaseSchema):
+    """Represent the common schema of DeploymentSettings for ImageBuilder and Cluster."""
+
+    lambda_functions_vpc_config = fields.Nested(
+        LambdaFunctionsVpcConfigSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return DeploymentSettings(**data)
+
+
 class BaseDevSettingsSchema(BaseSchema):
     """Represent the common schema of Dev Setting for ImageBuilder and Cluster."""
 
