@@ -293,6 +293,24 @@ def get_resource_name_from_resource_arn(resource_arn):
     return resource_arn.rsplit("/", 1)[-1] if resource_arn else ""
 
 
+def split_resource_prefix(resource_prefix):
+    if resource_prefix:
+        split_index = resource_prefix.rfind("/") + 1
+        return (
+            None
+            if split_index == 0
+            else resource_prefix
+            if split_index == len(resource_prefix)
+            else resource_prefix[:split_index],
+            None
+            if split_index == len(resource_prefix)
+            else resource_prefix
+            if split_index == 0
+            else resource_prefix[split_index:],
+        )
+    return None, None
+
+
 def grouper(iterable, size):
     """Slice iterable into chunks of size."""
     itr = iter(iterable)
@@ -361,3 +379,8 @@ def yaml_no_duplicates_constructor(loader, node, deep=False):
 def get_http_tokens_setting(imds_support):
     """Get http tokens settings for supported IMDS version."""
     return "required" if imds_support == "v2.0" else "optional"
+
+
+def remove_none_values(original_dictionary):
+    """Return a dictionary without entries with None value."""
+    return {key: value for key, value in original_dictionary.items() if value is not None}
