@@ -35,7 +35,13 @@ from pcluster.constants import (
     SUPPORTED_REGIONS,
     SUPPORTED_SCHEDULERS,
 )
-from pcluster.utils import get_installed_version, get_supported_os_for_architecture, get_supported_os_for_scheduler
+from pcluster.launch_template_utils import _LaunchTemplateBuilder
+from pcluster.utils import (
+    get_installed_version,
+    get_supported_os_for_architecture,
+    get_supported_os_for_scheduler,
+    remove_none_values,
+)
 from pcluster.validators.common import FailureLevel, Validator
 
 # pylint: disable=C0302
@@ -297,12 +303,12 @@ class MaxCountValidator(Validator):
 class EfaValidator(Validator):
     """Check if EFA and EFA GDR are supported features in the given instance type."""
 
-    def _validate(self, instance_type, efa_enabled, gdr_support):
+    def _validate(self, instance_type, efa_enabled, gdr_support, multiaz_enabled):
 
         instance_type_supports_efa = AWSApi.instance().ec2.get_instance_type_info(instance_type).is_efa_supported()
         if efa_enabled and not instance_type_supports_efa:
             self._add_failure(f"Instance type '{instance_type}' does not support EFA.", FailureLevel.ERROR)
-        if instance_type_supports_efa and not efa_enabled:
+        if instance_type_supports_efa and not efa_enabled and not multiaz_enabled:
             self._add_failure(
                 f"The EC2 instance selected ({instance_type}) supports enhanced networking capabilities using "
                 "Elastic Fabric Adapter (EFA). EFA enables you to run applications requiring high levels of "
@@ -317,12 +323,15 @@ class EfaValidator(Validator):
 class EfaPlacementGroupValidator(Validator):
     """Validate placement group if EFA is enabled."""
 
-    def _validate(self, efa_enabled: bool, placement_group_key: str, placement_group_disabled: bool):
-        if efa_enabled and placement_group_disabled:
+    def _validate(
+        self, efa_enabled: bool, placement_group_key: str, placement_group_disabled: bool, multi_az_enabled: bool
+    ):
+        # if multi_az is enabled suggestions about PlacementGroups will be suppressed
+        if efa_enabled and placement_group_disabled and not multi_az_enabled:
             self._add_failure(
                 "You may see better performance using a placement group for the queue.", FailureLevel.WARNING
             )
-        elif efa_enabled and placement_group_key is None:
+        elif efa_enabled and placement_group_key is None and not multi_az_enabled:
             self._add_failure(
                 "The placement group for EFA-enabled compute resources must be explicit. "
                 "You may see better performance using a placement group, but if you don't wish to use one please add "
@@ -374,6 +383,25 @@ class EfaSecurityGroupValidator(Validator):
         return False
 
 
+class EfaMultiAzValidator(Validator):
+    """Validate MultiAZ if EFA is enabled."""
+
+    def _validate(
+        self, queue_name: str, multi_az_enabled: bool, compute_resource_name: str, compute_resource_efa_enabled: bool
+    ):
+        if multi_az_enabled and compute_resource_efa_enabled:
+            message = (
+                f"You have enabled the Elastic Fabric Adapter (EFA) for the '{compute_resource_name}' Compute Resource"
+                f" on the '{queue_name}' queue. EFA is not supported across Availability zones. Either disable EFA "
+                "to use multiple subnets on the queue or specify only one subnet to enable EFA on "
+                "the compute resources."
+            )
+            self._add_failure(
+                message,
+                FailureLevel.ERROR,
+            )
+
+
 # --------------- Storage validators --------------- #
 
 
@@ -540,9 +568,9 @@ class ExistingFsxNetworkingValidator(Validator):
 
 class FsxArchitectureOsValidator(Validator):
     """
-    FSx networking validator.
+    FSx architecture and OS validator.
 
-    Validate file system mount point according to the head node subnet.
+    Validate that OS and architecture are compatible with FSx.
     """
 
     def _validate(self, architecture: str, os):
@@ -658,14 +686,64 @@ class NumberOfStorageValidator(Validator):
             )
 
 
+class ManagedFsxMultiAzValidator(Validator):
+    """
+    Managed FSx Storage Vs Multiple Subnets validator.
+
+    Validate if managed storage of type FSx is set when using multiple subnets in queues configuration.
+    """
+
+    def _validate(self, compute_subnet_ids, new_storage_count):
+        if len(compute_subnet_ids) > 1 and new_storage_count.get("fsx") > 0:
+            self._add_failure(
+                "Managed FSx storage created by ParallelCluster is not supported when specifying multiple subnet Ids "
+                "under the SubnetIds configuration of a queue. Please make sure to provide an existing FSx shared "
+                "storage, properly configured to work across the target subnets or remove the managed FSx storage to "
+                "use multiple subnets for a queue.",
+                FailureLevel.ERROR,
+            )
+
+
+class UnmanagedFsxMultiAzValidator(Validator):
+    """
+    Unmanaged FSx Storage Vs Multiple Subnets validator.
+
+    Unmanaged FSx volumes can exist in AZ that are different from the ones defined in queues configuration.
+    In these cases we notify customers that they may incur in increased latency and costs.
+    """
+
+    def _validate(self, queues, fsx_az_list):
+        for queue in queues:
+            queue_az_set = set(queue.networking.az_list)
+            fs_az_set = set(fsx_az_list)
+            # we want to ensure that all the az defined in the queue are supported by the FS
+            if not queue_az_set.issubset(fs_az_set):
+                self._add_failure(
+                    "Your configuration for Queue '{0}' includes multiple subnets and external shared storage "
+                    "configuration. Accessing a shared storage from different AZs can lead to increased storage "
+                    "networking latency and added inter-AZ data transfer costs.".format(queue.name),
+                    FailureLevel.INFO,
+                )
+
+
 class EfsIdValidator(Validator):  # TODO add tests
     """
     EFS id validator.
 
-    Validate if there are existing mount target in the head node availability zone
+    Validate if there are existing mount target in the cluster (head and computes) availability zone
     """
 
     def _validate(self, efs_id, avail_zones_mapping: dict, are_all_security_groups_customized):
+        availability_zones = avail_zones_mapping.keys()
+        if len(availability_zones) > 1 and not AWSApi.instance().efs.is_efs_standard(efs_id):
+            self._add_failure(
+                f"Cluster has subnets located in different availability zones but EFS ({efs_id}) uses OneZone EFS "
+                "storage class which works within a single Availability Zone. Please use subnets located in one "
+                "Availability Zone or use a standard storage class EFS.",
+                FailureLevel.ERROR,
+            )
+
+        avail_zones_missing_mount_target_for_efs_standard = []
         for avail_zone, subnets in avail_zones_mapping.items():
             head_node_target_id = AWSApi.instance().efs.get_efs_mount_target_id(efs_id, avail_zone)
             # If there is an existing mt in the az, need to check the inbound and outbound rules of the security groups
@@ -683,6 +761,17 @@ class EfsIdValidator(Validator):  # TODO add tests
                     )
                 if not are_all_security_groups_customized:
                     self._check_cidrs_cover_subnets(head_node_target_id, avail_zone, sg_ids, efs_id, subnets)
+            else:
+                if AWSApi.instance().efs.is_efs_standard(efs_id):
+                    avail_zones_missing_mount_target_for_efs_standard.append(avail_zone)
+        if avail_zones_missing_mount_target_for_efs_standard:
+            self._add_failure(
+                "There is no existing Mount Target for EFS '{0}' in these Availability Zones: '{1}'. "
+                "Please create an EFS Mount Target for those availability zones.".format(
+                    efs_id, avail_zones_missing_mount_target_for_efs_standard
+                ),
+                FailureLevel.ERROR,
+            )
 
     def _check_subnet_access(self, security_groups_ids, subnet_cidr, access_type):
         permission = "IpPermissions" if access_type == "in" else "IpPermissionsEgress"
@@ -941,6 +1030,10 @@ class MixedSecurityGroupOverwriteValidator(Validator):
 class _LaunchTemplateValidator(Validator):
     """Abstract class to contain utility functions used by head node and queue LaunchTemplate validators."""
 
+    def __init__(self):
+        super().__init__()
+        self._launch_template_builder = DictLaunchTemplateBuilder()
+
     @staticmethod
     def _build_launch_network_interfaces(
         network_interfaces_count, use_efa, security_group_ids, subnet, use_public_ips=False
@@ -966,7 +1059,7 @@ class _LaunchTemplateValidator(Validator):
     def _ec2_run_instance(self, availability_zone: str, **kwargs):  # noqa: C901 FIXME!!!
         """Wrap ec2 run_instance call. Useful since a successful run_instance call signals 'DryRunOperation'."""
         try:
-            AWSApi.instance().ec2.run_instances(**kwargs)
+            AWSApi.instance().ec2.run_instances(**remove_none_values(kwargs))
         except AWSClientError as e:
             code = e.error_code
             message = str(e)
@@ -1032,7 +1125,7 @@ class _LaunchTemplateValidator(Validator):
 class HeadNodeLaunchTemplateValidator(_LaunchTemplateValidator):
     """Try to launch the requested instance (in dry-run mode) to verify configuration parameters."""
 
-    def _validate(self, head_node, ami_id, tags):
+    def _validate(self, head_node, os, ami_id, tags):
         try:
             head_node_security_groups = []
             if head_node.networking.security_groups:
@@ -1057,6 +1150,10 @@ class HeadNodeLaunchTemplateValidator(_LaunchTemplateValidator):
                 NetworkInterfaces=head_node_network_interfaces,
                 DryRun=True,
                 TagSpecifications=self._generate_tag_specifications(tags),
+                KeyName=head_node.ssh.key_name,
+                BlockDeviceMappings=(
+                    self._launch_template_builder.get_block_device_mappings(head_node.local_storage.root_volume, os)
+                ),
             )
         except Exception as e:
             self._add_failure(
@@ -1087,7 +1184,7 @@ class HeadNodeImdsValidator(Validator):
 class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
     """Try to launch the requested instances (in dry-run mode) to verify configuration parameters."""
 
-    def _validate(self, queue, ami_id, tags):
+    def _validate(self, queue, os, ami_id, tags):
         try:
             # Retrieve network parameters
             queue_subnet_id = queue.networking.subnet_ids[0]
@@ -1097,9 +1194,6 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
             if queue.networking.additional_security_groups:
                 queue_security_groups.extend(queue.networking.additional_security_groups)
 
-            queue_placement_group_id = queue.networking.placement_group.id if queue.networking.placement_group else None
-            queue_placement_group = {"GroupName": queue_placement_group_id} if queue_placement_group_id else {}
-
             # Select the "best" compute resource to run dryrun tests against.
             # Resources with multiple NICs are preferred among others.
             # Temporarily limiting dryrun tests to 1 per queue to save boto3 calls.
@@ -1107,14 +1201,21 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
                 (compute_res for compute_res in queue.compute_resources if compute_res.max_network_interface_count > 1),
                 queue.compute_resources[0],
             )
+            compute_resource_placement_group = (
+                dry_run_compute_resource.networking.placement_group or queue.networking.placement_group
+            )
+
+            placement_group_name = compute_resource_placement_group.assignment
             # For SlurmFlexibleComputeResource test only the first InstanceType through a RunInstances
             self._test_compute_resource(
+                queue=queue,
+                os=os,
                 compute_resource=dry_run_compute_resource,
                 use_public_ips=bool(queue.networking.assign_public_ip),
                 ami_id=ami_id,
                 subnet_id=queue_subnet_id,
                 security_groups_ids=queue_security_groups,
-                placement_group=queue_placement_group,
+                placement_group={"GroupName": placement_group_name} if placement_group_name else {},
                 tags=tags,
             )
         except Exception as e:
@@ -1123,7 +1224,7 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
             )
 
     def _test_compute_resource(
-        self, compute_resource, use_public_ips, ami_id, subnet_id, security_groups_ids, placement_group, tags
+        self, queue, os, compute_resource, use_public_ips, ami_id, subnet_id, security_groups_ids, placement_group, tags
     ):
         """Test Compute Resource Instance Configuration."""
         network_interfaces = self._build_launch_network_interfaces(
@@ -1143,6 +1244,13 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
             NetworkInterfaces=network_interfaces,
             DryRun=True,
             TagSpecifications=self._generate_tag_specifications(tags),
+            InstanceMarketOptions=self._launch_template_builder.get_instance_market_options(queue, compute_resource),
+            CapacityReservationSpecification=self._launch_template_builder.get_capacity_reservation(
+                queue, compute_resource
+            ),
+            BlockDeviceMappings=self._launch_template_builder.get_block_device_mappings(
+                queue.compute_settings.local_storage.root_volume, os
+            ),
         )
 
 
@@ -1202,3 +1310,47 @@ class SchedulerValidator(Validator):
                 f"{scheduler} scheduler is not supported. Supported schedulers are: {', '.join(SUPPORTED_SCHEDULERS)}.",
                 FailureLevel.ERROR,
             )
+
+
+class DictLaunchTemplateBuilder(_LaunchTemplateBuilder):
+    """Concrete class to build a dict with EC2 run instance properties to simulate our launch templates."""
+
+    def _block_device_mapping_for_ebs(self, device_name, volume):
+        return {
+            "DeviceName": device_name,
+            "Ebs": remove_none_values(
+                {
+                    "Encrypted": volume.encrypted,
+                    "VolumeType": volume.volume_type,
+                    "Iops": volume.iops,
+                    "Throughput": volume.throughput,
+                    "DeleteOnTermination": volume.delete_on_termination,
+                    "VolumeSize": volume.size,
+                }
+            ),
+        }
+
+    def _block_device_mapping_for_virt(self, device_name, virtual_name):
+        return {"DeviceName": device_name, "VirtualName": virtual_name}
+
+    def _instance_market_option(self, market_type, spot_instance_type, instance_interruption_behavior, max_price):
+        return {
+            "MarketType": market_type,
+            "SpotOptions": remove_none_values(
+                {
+                    "SpotInstanceType": spot_instance_type,
+                    "InstanceInterruptionBehavior": instance_interruption_behavior,
+                    "MaxPrice": max_price,
+                }
+            ),
+        }
+
+    def _capacity_reservation(self, cr_target):
+        return {
+            "CapacityReservationTarget": remove_none_values(
+                {
+                    "CapacityReservationId": cr_target.capacity_reservation_id,
+                    "CapacityReservationResourceGroupArn": cr_target.capacity_reservation_resource_group_arn,
+                }
+            )
+        }
