@@ -36,6 +36,9 @@ class ValidationResult:
         self.level = level
         self.validator_type = validator_type
 
+    def __repr__(self):
+        return f"ValidationResult(level={self.level}, message={self.message})"
+
 
 class Validator(ABC):
     """Abstract validator. The children must implement the validate method."""
@@ -61,3 +64,10 @@ class Validator(ABC):
     def _validate(self, *args, **kwargs):
         """Must be implemented with specific validation logic."""
         pass
+
+
+class ValidatorContext:
+    """Context containing information about cluster environment meant to be passed to validators."""
+
+    def __init__(self, head_node_instance_id: str = None):
+        self.head_node_instance_id = head_node_instance_id
